{% extends "forms.html" %}
{% load forms %}


{% block title %}Start Application{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_application_list' %}">Applications</a>&nbsp;›
    Start application
</div>
{% endblock %}


{% block content %}
<div class="module">
    <h2>Start Application</h2>
    <form method="post" action=".">{% csrf_token %}
        <p>Please confirm you want to start a new application.</p>
        <div class="submit-row">
            <input type="submit" value="Go" class="default" />
        </div>
    </form>
</div>
{% endblock %}
