# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-02 18:08
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kgapplications', '0004_auto_20180301_1721'),
    ]

    operations = [
        migrations.AlterField(
            model_name='application',
            name='created_by',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
    ]
