{% extends "main.html" %}
{% load i18n %}

{% block title %}{% trans "XML-RPC Service" %}{% endblock %}

{% block content %}
<div id="content-main">
    <h2>{% trans "XML-RPC Service" %}</h2>

    <p>{% trans "You need to invoke this service using an XML-RPC Client." %}</p>

    <h3>{% trans "The following methods are available :" %}</h3>

    {% for m in methods %}
    <div class="functions">
      <h4>{{ m.0 }}</h4>
      <div class="function_desc">
        <strong>{% trans "Types of argument" %}{{ m.1.args|length|pluralize }} :</strong> {{ m.1.args }}
        <br />
        <strong>{% trans "Type of return" %} :</strong> {{ m.1.returns }}
        <br />
        <pre class="function_doc">{{ m.2 }}</pre>
      </div>
    </div>
    {% endfor %}
</div>
{% endblock %}

