{% extends "karaage/common/profile_base.html" %}
{% load software %}
{% load render_table from django_tables2 %}


{% block title %}Licensed Software - My profile{% endblock %}
{% block profile_title %}Licensed Software{% endblock %}


{% block profile_content %}
<div class="module">
    <h2>Software Agreements</h2>

    <div class="object-tools">
        <ul>
            <li><a href="{% url 'kg_software_list' %}" class="addlink">Add software</a></li>
        </ul>
    </div>

    {% get_software_license_agreement_table agreement_list as table %}
    {% render_table table %}
</div>
{% endblock %}
