# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

# Django 1.7 style AppConfig, see
# http://docs.djangoproject.com/en/1.7/ref/applications/
from .celery import app as celery_app  # NOQA


default_app_config = 'karaage.apps.Karaage'

__author__ = """Brian May"""
__email__ = 'brian@linuxpenguins.xyz'
__version__ = '5.0.14'
