# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-15 18:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('karaage', '0004_auto_20160429_0927'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='institutequota',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='institutequota',
            name='institute',
        ),
        migrations.RemoveField(
            model_name='institutequota',
            name='machine_category',
        ),
        migrations.AlterUniqueTogether(
            name='projectquota',
            unique_together=set([]),
        ),
        migrations.RemoveField(
            model_name='projectquota',
            name='machine_category',
        ),
        migrations.RemoveField(
            model_name='projectquota',
            name='project',
        ),
        migrations.RemoveField(
            model_name='account',
            name='machine_category',
        ),
        migrations.RemoveField(
            model_name='machine',
            name='category',
        ),
        migrations.DeleteModel(
            name='MachineCategory',
        ),
        migrations.DeleteModel(
            name='ProjectQuota',
        ),
        migrations.DeleteModel(
            name='InstituteQuota',
        ),
    ]
