body {
    margin: 0;
    padding: 0;
    font-size: 11px;
    font-family: "Lucida Grande","DejaVu Sans","Bitstream Vera Sans",Verdana,Arial,sans-serif;
    color: #333;
    background: #fff;
}


/* LINKS */

a:link, a:visited {
    color: #5b80b2;
    text-decoration: none;
}

a:hover {
    color: #036;
}

a img {
    border: none;
}

a.section:link, a.section:visited {
    color: white;
    text-decoration: none;
}


/* GLOBAL DEFAULTS */

p, ol, ul, dl {
    margin: .2em 0 .8em 0;
}

p {
    padding: 0;
    line-height: 140%;
}

h1,h2,h3,h4,h5 {
    font-weight: bold;
}

h1 {
    font-size: 18px;
    color: #666;
    padding: 0 6px 0 0;
    margin: 0 0 .2em 0;
}

h2 {
    font-size: 16px;
    margin: 1em 0 .5em 0;
}

h3 {
    font-size: 14px;
    margin: .8em 0 .3em 0;
    color: #666;
    font-weight: bold;
}

h4 {
    font-size: 12px;
    margin: 1em 0 .8em 0;
    padding-bottom: 3px;
}

h5 {
    font-size: 10px;
    margin: 1.5em 0 .5em 0;
    color: #666;
    text-transform: uppercase;
    letter-spacing: 1px;
}

ul li {
    list-style-type: square;
    padding: 1px 0;
}

ul.plainlist li {
    list-style-type: none;
}

li.selected {
    font-weight: bold;
}

li ul {
    margin-bottom: 0;
}

dt {
    font-weight: bold;
    margin-top: 4px;
}

dd {
    margin-left: 0;
}

blockquote {
    color: #777;
    margin-left: 2px;
    padding-left: 10px;
    border-left: 5px solid #ddd;
}

code, pre {
    font-family: "Bitstream Vera Sans Mono", Monaco, "Courier New", Courier, monospace;
    background: inherit;
    color: #666;
}

code strong {
    color: #930;
}

hr {
    clear: both;
    color: #eee;
    background-color: #eee;
    height: 1px;
    border: none;
    margin: 0;
    padding: 0;
    line-height: 1px;
}

/* TEXT STYLES & MODIFIERS */

.help, p.help {
    font-size: 10px !important;
    color: #999;
}

p img, h1 img, h2 img, h3 img, h4 img, td img {
    vertical-align: middle;
}

.quiet strong {
    font-weight: bold !important;
}

.float-right {
    float: right;
}

.float-left {
    float: left;
}

.clear {
    clear: both;
}

.example {
    margin: 10px 0;
    padding: 5px 10px;
    background: #efefef;
}

/* TABLES */

table {
    border-collapse: collapse;
    border-color: #ccc;
    overflow: auto;
    width: 100%;
}

td, th {
    line-height: 13px;
    border: 1px solid #ddd;
    vertical-align: top;
    padding: 5px;
    font-family: "Lucida Grande", Verdana, Arial, sans-serif;
}

table.keyvalue th {
    width: 100px;
}

th {
    text-align: left;
    font-weight: bold;
    color: #333;
    background: #7CA0A0;
}

tbody tr:nth-child(even) {background: #DDD}
tbody tr:nth-child(odd) {background: #FFF}

tfoot td {
    font-weight: bold;
}

table p {
    padding: 0;
}

table thead th.orderable > a {
    padding-right: 20px;
}
table thead th.orderable.asc > a {
    background: url(../img/arrow-active-down.png) right center no-repeat;
}
table thead th.orderable.desc > a {
    background: url(../img/arrow-active-up.png) right center no-repeat;
}

.yes {
    padding: 0 0 0 16px;
    background: url(../img/icon-yes.gif) 0 .25em no-repeat;
}

.no {
    padding: 0 0 0 16px;
    background: url(../img/icon-no.gif) 0 .25em no-repeat;
}

.locked {
    padding: 5px 0 5px 16px;
    background: url(../img/lock.png) 0 .25em no-repeat;
}



/* FORM DEFAULTS */

input, textarea, select, .form-row p {
    margin: 2px 0;
    padding: 2px 3px;
    vertical-align: middle;
    font-family: "Lucida Grande", Verdana, Arial, sans-serif;
    font-weight: normal;
}

textarea {
    vertical-align: top !important;
}

/* FORM BUTTONS */

.button, input[type=submit], input[type=button] {
    background: white url(../img/nav-bg.gif) bottom repeat-x;
    padding: 3px 5px;
    color: black;
    border: 1px solid #bbb;
    border-color: #ddd #aaa #aaa #ddd;
}

.button:active, input[type=submit]:active, input[type=button]:active {
    background: url(../img/nav-bg-reverse.gif) top;
}

.button[disabled], input[type=submit][disabled], input[type=button][disabled] {
    background: url(../img/nav-bg.gif) bottom;
    opacity: 0.4;
}

.button.default, input[type=submit].default, input[type=button].default {
    border: 2px solid #5b80b2;
    background: #7CA0C7 url(../img/default-bg.gif) bottom repeat-x;
    font-weight: bold;
    color: white;
}

.button.default:active, input[type=submit].default:active, input[type=button]:active {
    background: url(../img/default-bg-reverse.gif) top;
}

.button[disabled].default, input[type=submit][disabled].default, input[type=button][disabled].default {
    background: url(../img/default-bg.gif) bottom;
    opacity: 0.4;
}


/* SUBMIT ROW */

.submit-row .button.default, .submit-row input.default {
    float: right;
}


/* MODULES */

.module, .module.object-tools {
    border: 1px solid #666;
    margin: 0 0 10px 0;
    background: white;
    width: 100%;
}

.module p, .module ul, .module h3, .module h4, .module dl, .module pre, .module form, .module .table-container, .module .object-tools {
    margin: 0 4px 2px 4px;
}

.module table {
}

.module blockquote {
    margin-left: 12px;
}

.module h2, .module caption, .inline-group h2 {
    margin-top: 0;
    margin-bottom: 4px;
    padding: 2px 5px 3px 5px;
    font-size: 11px;
    text-align: left;
    font-weight: bold;
    background: #7CA0C7;
    color: #666;
}

.module h3 {
    margin-top: 10px;
    margin-bottom: 2px;
    padding: 2px 5px 3px 5px;
    font-size: 11px;
    text-align: left;
    font-weight: bold;
    background-color: #C7C7A0;
}



/* MESSAGES & ERRORS */

ul.messagelist {
    padding: 0 0 5px 0;
    margin: 0;
}

ul.messagelist li {
    display: block;
    padding: 4px 5px 4px 25px;
    margin: 0 0 3px 0;
    border-bottom: 1px solid #ddd;
    color: #666;
    background: #ffc url(../img/icon_success.gif) 5px .3em no-repeat;
}

ul.messagelist li.warning{
    background-image: url(../img/icon_alert.gif);
}

ul.messagelist li.error{
    background-image: url(../img/icon_error.gif);
}

.errornote {
    display: block;
    padding: 4px 5px 4px 25px;
    margin: 0 0 3px 0;
    border: 1px solid red;
    color: red;
    background: #ffc url(../img/icon_error.gif) 5px .3em no-repeat;
}

ul.errorlist {
    margin: 0 !important;
    padding: 0 !important;
}

.errorlist li {
    display: block;
    padding: 4px 5px 4px 25px;
    margin: 0 0 3px 0;
    border: 1px solid red;
    color: white;
    background: red url(../img/icon_alert.gif) 5px .3em no-repeat;
}

.errorlist li a {
 	color: white;
    text-decoration: underline;
}

td ul.errorlist {
    margin: 0 !important;
    padding: 0 !important;
}

td ul.errorlist li {
    margin: 0 !important;
}

.errors {
    background: #ffc;
}

.errors input, .errors select, .errors textarea {
    border: 1px solid red;
}

.description {
    padding: 5px 0 0 12px;
}


/* BREADCRUMBS */

div.breadcrumbs {
    background: white url(../img/nav-bg-reverse.gif) 0 -10px repeat-x;
    padding: 2px 8px 3px 8px;
    color: #999;
    border-top: 1px solid white;
    border-bottom: 1px solid #ccc;
    text-align: left;
}


/* ACTION ICONS */

.addlink {
    padding-left: 16px;
    background: url(../img/icon_addlink.gif) 0 .2em no-repeat;
}

.changelink {
    padding-left: 16px;
    background: url(../img/icon_changelink.gif) 0 .2em no-repeat;
}

.deletelink {
    padding-left: 16px;
    background: url(../img/icon_deletelink.gif) 0 .25em no-repeat;
}

.yeslink {
    padding-left: 16px;
    background: url(../img/icon-yes.gif) 0 .25em no-repeat;
}

.nolink {
    padding-left: 16px;
    background: url(../img/icon-no.gif) 0 .25em no-repeat;
}

.locklink {
    padding-left: 16px;
    background: url(../img/lock.png) 0 .25em no-repeat;
}

.unlocklink {
    padding-left: 16px;
    background: url(../img/unlock.png) 0 .25em no-repeat;
}

/* OBJECT TOOLS */

.object-tools {
    display: inline-block;
    border: 1px solid black;
    margin-bottom: 4px;
}

.object-tools ul {
    margin: 0;
    padding: 2px 4px 0 4px;
}

.object-tools li {
    display: inline-block;
    vertical-align: bottom;
    margin-bottom: 2px;
}

.object-tools a {
    display: block;
    width: 100px;
    height: 30px;
    padding: 4px 4px 4px 20px;
    vertical-align: bottom;
    color: #333;
    background: #bbb 4px 4px;
}

.object-tools a.active {
    background-color: #C7C7A0;
}

.object-tools a:hover {
    color: #fff;
    background-color: #333;
}


#content-related .object-tools li {
    width: 100%;
}

#content-related .object-tools a {
    width: auto;
    color: #333;
    background-color: #eee;
}

#content-related .object-tools a.active {
    background-color: #aaa;
}


#content-related .object-tools a:hover {
    width: auto;
    color: #fff;
    background-color: #333;
}


/* PAGE STRUCTURE */

#container {
    position: relative;
    width: 100%;
    min-width: 760px;
    padding: 0;
}

#content {
    margin: 10px 15px;
}

#header {
    width: 100%;
}

#content-main {
    float: left;
    width: 100%;
}

#content-related {
    float: right;
    width: 16em;
    position: relative;
    margin-right: -17em;
}

#footer {
    clear: both;
    padding: 10px;
}

/* COLUMN TYPES */

.colMS {
    margin-right: 18em !important;
}

.colSM {
    margin-left: 18em !important;
}

.colSM #content-related {
    float: left;
    margin-right: 0;
    margin-left: -17em;
}

.colSM #content-main {
    float: right;
}

.popup .colM {
    width: 95%;
}

.dashboard #content {
    width: 500px;
}

/* HEADER */

#header {
    background: #417690;
    color: #ffc;
    overflow: hidden;
}

#header a:link, #header a:visited {
    color: white;
}

#header a:hover {
    text-decoration: underline;
}

#branding h1 {
    padding: 0 10px;
    font-size: 18px;
    margin: 8px 0;
    font-weight: normal;
    color: #f4f379;
}

#branding h2 {
    padding: 0 10px;
    font-size: 14px;
    margin: -8px 0 8px 0;
    font-weight: normal;
    color: #ffc;
}

#user-tools {
    position: absolute;
    top: 0;
    right: 0;
    padding: 1.2em 10px;
    font-size: 11px;
    text-align: right;
}

/* SIDEBAR */

#content-related input[type="text"] {
    width: 9em;
}
#content-related input[type="submit"] {
    width: 2em;
    padding: 0;
}

#content-related .module h2 {
    background: #C7C7A0;
    color: #666;
}

/* OTHER */

.graph_key_red { background-color: red; }
.graph_key_blue { background-color: blue; }
.graph_key_green { background-color: green; }
.graph_key_yellow { background-color: yellow; }
.graph_key_orange { background-color: orange; }
.graph_key_cyan { background-color: cyan; }
.graph_key_magenta { background-color: magenta; }
.graph_key_purple { background-color: purple; }
.graph_key_grey { background-color: grey; }
.graph_key_pink { background-color: pink; }

.hidden { display: none !important; }


/* TOOLBAR */

.toolbar form {
    display: inline-block;
}

.toolbar ul {
    display: inline-block;
}

.toolbar .object-tools {
    float: right;
}

/* django-tables pagination */

.pagination {
    font-size: 11px;
    line-height: 22px;
    margin: 0;
    border-top: 1px solid #666;
    padding: 10px 0 10px 0;
} 

.pagination li {
    display: inline;
    padding: 2px 6px;
    margin: 2px;
}

.pagination li.link {
    border: solid 1px #ccc;
    background: white;
    text-decoration: none;
}

/* DIALOG */

#dialog-modal {
    display: none;
}
