{% extends "forms.html" %}


{% block title %}Lock person{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_person_list' %}">People</a>&nbsp;›
    <a href="{% url 'kg_person_detail' person.username %}">{{ person }}</a>&nbsp;›
    Lock
</div>
{% endblock %}


{% block content %}
<h1>Are you sure?</h1>
<p>Are you sure you want to lock the person "{{ person }}"? </p>
<form method="post" action=".">{% csrf_token %}
<div>
    <input type="hidden" name="post" value="yes" />
    <input type="submit" value="Yes, I'm sure" />
</div>
</form>
{% endblock %}
