{% extends "main.html" %}
{% load render_table from django_tables2 %}
{% load forms %}
{% load staticfiles %}


{% block title %}Institutes{% endblock %}
{% block short_title %}Institute List{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    Institutes
</div>
{% endblock %}


{% block content %}
<div id="content-main">
    <div class="toolbar">
        <button id="opener">Filter</button>
        {% for key, value in spec %}
            {{ key }} = {{ value }}{% if not forloop.last %}, {% else %}.{% endif %}
        {% endfor %}
        {% if is_admin %}
        <div class="object-tools">
            <ul>
                <li><a href="{% url 'kg_institute_add' %}" class="addlink">Add institute</a></li>
            </ul>
        </div>
        {% endif %}
    </div>
    <div class="module clear">
        <h2>{{ title }}</h2>
        {% render_table table %}
    </div>
    <form action="" id="dialog-modal" title="Filter" method="get">
        {% form_as_div filter.form %}
    </form>
</div>
{% endblock %}
