{% extends "forms.html" %}


{% block title %}Delete Project{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_project_list' %}">Projects</a>&nbsp;›
    <a href="{% url 'kg_project_detail' project.id %}">{{ project.pid }}</a>&nbsp;›
    Delete
</div>
{% endblock %}


{% block content %}
{% if error %}
    <h1>Cannot delete project</h1>
    <p>{{ error }}</p>
{% else %}
    <h1>Are you sure?</h1>    
    <p>Are you sure you want to delete the project "{{ project }}"? </p>
    <form method="post" action=".">{% csrf_token %}
        <div>
            <input type="hidden" name="post" value="yes" />
            <input type="submit" value="Yes, I'm sure" />
        </div>
    </form>
{% endif %}
{% endblock %}
