{% extends "main.html" %}
{% load render_table from django_tables2 %}


{% block title %}Bounced Email{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_person_list' %}">People</a>&nbsp;›
    <a href="{% url 'kg_person_detail' person.username %}">{{ person }}</a>&nbsp;›
    Bounced Email
</div>
{% endblock %}


{% block content %}
<div id="content-main">
    <p>Submitting this form will send an email to all the <b>active</b> project leaders below and also lock the account</p>

    {% render_table leader_list %}

    <form action="." method="post">{% csrf_token %}
        <div>
            <input name="post" type="hidden" value="yes"/>
            <input type="submit" value="Yes, I'm sure"/>
        </div>
    </form>
</div>
{% endblock %}
