{% extends "main.html" %}


{% block title %}Institute (Verbose){% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_institute_list' %}">Groups</a>&nbsp;›
    <a href="{% url 'kg_institute_detail' institute.pk %}">{{ institute }}</a>&nbsp;›
    Verbose
</div>
{% endblock %}


{% block content %}
<div id="content-main">

    <div class="module">
        <h2>Institute details - {{institute}}</h2>
        {% for a in institute_details %}
        <h3>{{a.datastore}}</h3>
        <div class="table-container">
            <table class="keyvalue">
                {% for i,j in a.items %}
                    <tr>
                        <th>{{ i }}</th><td>{{ j }}</td>
                   </tr>
                {% endfor %}
            </table>
        </div>
        {% empty %}
        <p>No datastores.</p>
        {% endfor %}
    </div>

</div>
{% endblock %}
