{% extends "base.html" %}
{% load i18n %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    {% trans "Server error" %}
</div>
{% endblock %}

{% block title %}{% trans 'Server error (500)' %}{% endblock %}
{% block userlinks %}{% endblock %}

{% block site_name_title %}{% trans 'Karaage' %}{% endblock %}

{% block branding %}
<h1 id="site-name">{% trans 'Karaage' %}</h1>
{% endblock %}

{% block content %}
<h1>{% trans 'Server Error <em>(500)</em>' %}</h1>
<p>{% trans "There's been an error. It's been reported to the site administrators via e-mail and should be fixed shortly. Thanks for your patience." %}</p>
{% endblock %}
