<form id="changelist-search" action="." method="get">
    <label for="search"><img src="{{ STATIC_URL }}img/icon_searchbox.png" alt="Search" /></label>
    <input type="text" size="40" name="search" value="{{ terms }}" id="search" />
    <input type="submit" value="Go" />
</form>
<script type="text/javascript">document.getElementById("search").focus();</script>
