{% extends "karaage/common/profile_base.html" %}
{% load karaage_tags %}
{% load forms %}{% load filters %}


{% block title %}My profile{% endblock %}
{% block profile_title %}My profile{% endblock %}


{% block profile_content %}
<div class="module">
    <h2>My Profile</h2>

    <p>Welcome {{ person }} to Karaage. Please select from the links.</p>

    <div class="object-tools">
        <ul>
            {% if user.is_authenticated %}
            <li><a class="{% active request "profile/personal/" %}" href="{% url 'kg_profile_personal' %}">Personal</a></li>
            {% if SHIB_SUPPORTED %}
            <li><a class="{% active request "profile/saml/" %}" href="{% url 'kg_profile_saml' %}">SAML Profile</a></li>
            {% endif %}
            <li><a class="{% active request "profile/accounts/" %}" href="{% url 'kg_profile_accounts' %}">Accounts</a></li>
            <li><a class="{% active request "profile/institutes/" %}" href="{% url 'kg_profile_institutes' %}">Institutes</a></li>
            <li><a class="{% active request "profile/projects/" %}" href="{% url 'kg_profile_projects' %}">Projects</a></li>
            {% endif %}
            {% for_each_app_include "main_profile.html" %}
        </ul>
    </div>
</div>
{% endblock %}
