{% extends "forms.html" %}


{% block title %}Add Person to Group{% endblock %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_group_list' %}">Groups</a>&nbsp;›
    <a href="{% url 'kg_group_detail' group.name %}">{{ group }}</a>&nbsp;›
    Add person
</div>
{% endblock %}



{% block content %}
<div class="module">
    <h2>Add Person</h2>
    <form method="post" action="." class="simple">{% csrf_token %}
        {{ form }}
        <input type="submit" value="Add" class="default" />
    </form>
</div>
{% endblock %}
