{% extends "main.html" %}


{% block breadcrumbs %}
<div class="breadcrumbs">
    <a href='{% url "index" %}'>Home</a>&nbsp;›
    <a href="{% url 'kg_person_list' %}">People</a>&nbsp;›
    <a href="{% url 'kg_person_detail' person.username %}">{{ person }}</a>&nbsp;›
    Password reset
</div>
{% endblock %}

{% block title %}Password reset successful{% endblock %}

{% block content %}
<div id="content-main">

<h1>Password reset successful</h1>

<p>We've e-mailed {{ person }} instructions for setting their password. They should be receiving it shortly.</p>

</div>
{% endblock %}
